@@ -1,11 +1,12 @@
 package App::CryptoCurrencyUtils;
 
-our $DATE = '2018-01-30'; # DATE
-our $VERSION = '0.007'; # VERSION
+our $DATE = '2018-01-31'; # DATE
+our $VERSION = '0.008'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
+use Log::ger;
 
 our %SPEC;
 
@@ -28,6 +29,16 @@ our %arg_coins = (
     },
 );
 
+our %arg_coins_opt = (
+    coins => {
+        'x.name.is_plural' => 1,
+        'x.name.singular' => 'coin',
+        schema => ['array*', of=>'cryptocurrency::symbol_or_name*'],
+        pos => 0,
+        greedy => 1,
+    },
+);
+
 our %arg_exchange = (
     exchange => {
         schema => 'cryptoexchange::name*',
@@ -47,14 +58,134 @@ our %arg_exchanges = (
     },
 );
 
-$SPEC{coin_cmc} = {
+our %arg_convert = (
+    convert => {
+        schema => ['str*', in=>["AUD", "BRL", "CAD", "CHF", "CLP", "CNY", "CZK", "DKK", "EUR", "GBP", "HKD", "HUF", "IDR", "ILS", "INR", "JPY", "KRW", "MXN", "MYR", "NOK", "NZD", "PHP", "PKR", "PLN", "RUB", "SEK", "SGD", "THB", "TRY", "TWD", "ZAR"]],
+    },
+);
+
+sub _get_json {
+    require HTTP::Tiny;
+    require JSON::MaybeXS;
+
+    my ($url) = @_;
+
+    my $res = HTTP::Tiny->new->get($url);
+    return [$res->{status}, $res->{reason}] unless $res->{success};
+
+    my $data;
+    eval { $data = JSON::MaybeXS::decode_json($res->{content}) };
+    return [500, "Can't decode JSON: $@"] if $@;
+
+    [$res->{status}, $res->{reason}, $data];
+}
+
+sub _get_json_cmc {
+    my $url = shift;
+    my $res = _get_json($url);
+
+    {
+        last unless $res->[0] == 200;
+        if (ref($res) eq 'HASH' && $res->{error}) {
+            $res = [500, "Got error response from CMC API: $res->{error}"];
+            last;
+        }
+    }
+    $res;
+}
+
+$SPEC{coin_cmc_summary} = {
     v => 1.1,
-    summary => "Go to coin's CMC (coinmarketcap.com) currency page",
+    summary => "Get coin's CMC (coinmarketcap.com) summary",
+    description => <<'_',
+
+Currently retrieves https://api.coinmarketcap.com/v1/ticker/<coin-id>/ and
+return the data in a table.
+
+If no coins are specified, will return global data.
+
+_
+    args => {
+        %arg_coins_opt,
+        %arg_convert,
+    },
+};
+sub coin_cmc_summary {
+    require CryptoCurrency::Catalog;
+
+    my %args = @_;
+
+    my $cat = CryptoCurrency::Catalog->new;
+
+    unless ($args{coins} && @{ $args{coins} }) {
+        return global_cmc_summary();
+    }
+
+    my @rows;
+  CURRENCY:
+    for my $cur0 (@{ $args{coins} }) {
+
+        my $cur;
+        {
+            eval { $cur = $cat->by_symbol($cur0) };
+            last if $cur;
+            eval { $cur = $cat->by_name($cur0) };
+            last if $cur;
+            warn "No such cryptocurrency symbol/name '$cur0'";
+            next CURRENCY;
+        }
+
+        my $res = _get_json_cmc(
+            "https://api.coinmarketcap.com/v1/ticker/$cur->{safename}/".
+                ($args{convert} ? "?convert=$args{convert}" : ""));
+        unless ($res->[0] == 200) {
+            log_error("Can't get API result for $cur->{name}: $res->[0] - $res->[1]");
+            next CURRENCY;
+        }
+        delete $res->[2][0]{id};
+        push @rows, $res->[2][0];
+    }
+
+    my $resmeta = {
+        'table.fields' => [qw/symbol name rank price_usd price_btc/],
+    };
+
+    [200, "OK", \@rows, $resmeta];
+}
+
+$SPEC{global_cmc_summary} = {
+    v => 1.1,
+    summary => "Get global CMC (coinmarketcap.com) summary",
+    description => <<'_',
+
+Currently retrieves https://api.coinmarketcap.com/v1/ticker/<coin-id>/ and
+
+_
+    args => {
+        %arg_convert,
+    },
+};
+sub global_cmc_summary {
+    my %args = @_;
+
+    my $res = _get_json_cmc(
+        "https://api.coinmarketcap.com/v1/global/".
+            ($args{convert} ? "?convert=$args{convert}" : ""));
+    unless ($res->[0] == 200) {
+        return [500, "Can't get API result: $res->[0] - $res->[1]"];
+    }
+
+    [200, "OK", $res->[2]];
+}
+
+$SPEC{open_coin_cmc} = {
+    v => 1.1,
+    summary => "Open coin's CMC (coinmarketcap.com) currency page in the browser",
     args => {
         %arg_coins,
     },
 };
-sub coin_cmc {
+sub open_coin_cmc {
     require CryptoCurrency::Catalog;
     my %args = @_;
 
@@ -81,9 +212,9 @@ sub coin_cmc {
     [200];
 }
 
-$SPEC{coin_mno} = {
+$SPEC{open_coin_mno} = {
     v => 1.1,
-    summary => "Go to coin's MNO (masternodes.online) currency page",
+    summary => "Open coin's MNO (masternodes.online) currency page in the browser",
     description => <<'_',
 
 Currently does not perform any translation between CMC -> MNO currency code if
@@ -94,7 +225,7 @@ _
         %arg_coins,
     },
 };
-sub coin_mno {
+sub open_coin_mno {
     require CryptoCurrency::Catalog;
     require URI::Escape;
 
@@ -124,14 +255,14 @@ sub coin_mno {
     [200];
 }
 
-$SPEC{exchange_cmc} = {
+$SPEC{open_exchange_cmc} = {
     v => 1.1,
-    summary => "Go to exchange's CMC (coinmarketcap.com) exchange page",
+    summary => "Open exchange's CMC (coinmarketcap.com) exchange page in the browser",
     args => {
         %arg_exchanges,
     },
 };
-sub exchange_cmc {
+sub open_exchange_cmc {
     require CryptoExchange::Catalog;
     my %args = @_;
 
@@ -208,7 +339,7 @@ App::CryptoCurrencyUtils - CLI utilities related to cryptocurrencies
 
 =head1 VERSION
 
-This document describes version 0.007 of App::CryptoCurrencyUtils (from Perl distribution App-CryptoCurrencyUtils), released on 2018-01-30.
+This document describes version 0.008 of App::CryptoCurrencyUtils (from Perl distribution App-CryptoCurrencyUtils), released on 2018-01-31.
 
 =head1 DESCRIPTION
 
@@ -216,11 +347,9 @@ This distribution contains the following CLI utilities:
 
 =over
 
-=item * L<coin-cmc>
+=item * L<coin-cmc-summary>
 
-=item * L<coin-mno>
-
-=item * L<exchange-cmc>
+=item * L<global-cmc-summary>
 
 =item * L<grep-coin>
 
@@ -230,18 +359,29 @@ This distribution contains the following CLI utilities:
 
 =item * L<list-exchanges>
 
+=item * L<open-coin-cmc>
+
+=item * L<open-coin-mno>
+
+=item * L<open-exchange-cmc>
+
 =back
 
 =head1 FUNCTIONS
 
 
-=head2 coin_cmc
+=head2 coin_cmc_summary
 
 Usage:
 
- coin_cmc(%args) -> [status, msg, result, meta]
+ coin_cmc_summary(%args) -> [status, msg, result, meta]
 
-Go to coin's CMC (coinmarketcap.com) currency page.
+Get coin's CMC (coinmarketcap.com) summary.
+
+Currently retrieves https://api.coinmarketcap.com/v1/ticker/<coin-id>/ and
+return the data in a table.
+
+If no coins are specified, will return global data.
 
 This function is not exported.
 
@@ -249,7 +389,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<coins>* => I<array[cryptocurrency::symbol_or_name]>
+=item * B<coins> => I<array[cryptocurrency::symbol_or_name]>
+
+=item * B<convert> => I<str>
 
 =back
 
@@ -265,16 +407,15 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 coin_mno
+=head2 global_cmc_summary
 
 Usage:
 
- coin_mno(%args) -> [status, msg, result, meta]
+ global_cmc_summary(%args) -> [status, msg, result, meta]
 
-Go to coin's MNO (masternodes.online) currency page.
+Get global CMC (coinmarketcap.com) summary.
 
-Currently does not perform any translation between CMC -> MNO currency code if
-there is a difference.
+Currently retrieves https://api.coinmarketcap.com/v1/ticker/<coin-id>/ and
 
 This function is not exported.
 
@@ -282,37 +423,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<coins>* => I<array[cryptocurrency::symbol_or_name]>
-
-=back
-
-Returns an enveloped result (an array).
-
-First element (status) is an integer containing HTTP status code
-(200 means OK, 4xx caller error, 5xx function error). Second element
-(msg) is a string containing error message, or 'OK' if status is
-200. Third element (result) is optional, the actual result. Fourth
-element (meta) is called result metadata and is optional, a hash
-that contains extra information.
-
-Return value:  (any)
-
-
-=head2 exchange_cmc
-
-Usage:
-
- exchange_cmc(%args) -> [status, msg, result, meta]
-
-Go to exchange's CMC (coinmarketcap.com) exchange page.
-
-This function is not exported.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<exchanges>* => I<array[cryptoexchange::name]>
+=item * B<convert> => I<str>
 
 =back
 
@@ -381,6 +492,99 @@ that contains extra information.
 
 Return value:  (any)
 
+
+=head2 open_coin_cmc
+
+Usage:
+
+ open_coin_cmc(%args) -> [status, msg, result, meta]
+
+Open coin's CMC (coinmarketcap.com) currency page in the browser.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<coins>* => I<array[cryptocurrency::symbol_or_name]>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 open_coin_mno
+
+Usage:
+
+ open_coin_mno(%args) -> [status, msg, result, meta]
+
+Open coin's MNO (masternodes.online) currency page in the browser.
+
+Currently does not perform any translation between CMC -> MNO currency code if
+there is a difference.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<coins>* => I<array[cryptocurrency::symbol_or_name]>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 open_exchange_cmc
+
+Usage:
+
+ open_exchange_cmc(%args) -> [status, msg, result, meta]
+
+Open exchange's CMC (coinmarketcap.com) exchange page in the browser.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<exchanges>* => I<array[cryptoexchange::name]>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/App-CryptoCurrencyUtils>.
